unit Simulation;

{$mode objfpc}{$H+}

interface

uses
  Physics, Forces, fgl;

type
  TBodyOwnerList = class(specialize TFPGObjectList<TPhysicsBody>);
  TForcesOwnerList = class(specialize TFPGObjectList<TForce>);

  { TSimulation }

  TSimulation = class
  private
    FBodies: TBodyOwnerList;
    FForces: TForcesOwnerList;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Step(const Delta: Double);

    procedure AddBody(Body: TPhysicsBody);
    procedure AddForce(Force: TForce);
  end;

implementation

{ TSimulation }

constructor TSimulation.Create;
begin
  FForces := TForcesOwnerList.Create;
  FBodies := TBodyOwnerList.Create;
end;

destructor TSimulation.Destroy;
begin
  FForces.Free;
  FBodies.Free;
  inherited Destroy;
end;

procedure TSimulation.Step(const Delta: Double);
var
  force: TForce;
  body: TPhysicsBody;
begin
  for force in FForces do
    force.Update(Delta);
  for body in FBodies do
  begin
    body.ResetForces;
    for force in FForces do
      body.ApplyForce(force[body]);
  end;
  for body in FBodies do
    body.Update(Delta);
end;

procedure TSimulation.AddBody(Body: TPhysicsBody);
begin
  FBodies.Add(Body);
end;

procedure TSimulation.AddForce(Force: TForce);
begin
  FForces.Add(Force);
end;

end.
