unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
  Buttons, ComCtrls, Simulation, Physics, Forces;

type

  { TForm1 }

  TForm1 = class(TForm)
    EarthShape: TShape;
    DeltaBox: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    MoonShape: TShape;
    StartStopButton: TSpeedButton;
    Timer1: TTimer;
    TrackBar1: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure StartStopButtonClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
  private
    Sim: TSimulation;
    Earth: TSphereBody;
    Moon: TSphereBody;
    Gravity: TNewtonGravity;
    Drag: TDrag;

    function CoordinatesToPixel(pos: TVector3): TPoint;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.StartStopButtonClick(Sender: TObject);
begin
  Timer1.Enabled:=not Timer1.Enabled;
end;

procedure TForm1.Timer1Timer(Sender: TObject);    
const runtime: Double = 0;
var
  EarthPos, MoonPos: TPoint;
  delta: Double;
begin
  delta := String(DeltaBox.Text).ToDouble;
  Sim.Step(delta);
  runtime += delta;
  EarthPos := CoordinatesToPixel(Earth.Postion);
  MoonPos := CoordinatesToPixel(Moon.Postion);
  EarthShape.Left:=EarthPos.x - EarthShape.Width div 2;
  EarthShape.Top:=EarthPos.y - EarthShape.Height div 2;
  MoonShape.Left:=MoonPos.x - MoonShape.Width div 2;
  MoonShape.Top:=MoonPos.y - MoonShape.Height div 2;
  Caption:=round(runtime/(60*60*24)).ToString + ' Days';
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  Drag.Roh:=TrackBar1.Position / 1000;
end;

function TForm1.CoordinatesToPixel(pos: TVector3): TPoint;
begin
  Result.x := round(pos.X / 5e6) + ClientWidth div 2;
  Result.y := round(pos.Y / 5e6) + ClientHeight div 2;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  // Data from https://nssdc.gsfc.nasa.gov/planetary/factsheet/moonfact.html
  Sim := TSimulation.Create;
  Earth := TSphereBody.Create(6.3e6, 5.9e24, TVector3.Zero, TVector3.Zero);
  Moon := TSphereBody.Create(1.7e6, 7.3e22, Vector2(0, -3.8e8), Vector2(1022, 0));
  Gravity := TNewtonGravity.Create;
  Gravity.ReferenceBodies.Add(Moon);
  Gravity.ReferenceBodies.Add(Earth);
  Drag := TDrag.Create(0);
  Sim.AddForce(Gravity);
  Sim.AddForce(Drag);
  Sim.AddBody(Earth);
  Sim.AddBody(Moon);
end;

end.

